; NAVTEX.ASM  --  Einfacher Navtex-Empfaenger             11-AUG-00 / 01-OCT-00
;
; Schaltungsbeschreibung siehe FUNKAMATEUR Heft 11/2000 und 12/2000.
;
; Klaus Betke, Am Entengrund 7, D-26160 Bad Zwischenahn, Email: betke@itap.de
;
; WICHTIGER HINWEIS: Dieses Programm wurde sorgfaeltig getestet. Dennoch sind
; Fehler nicht auszuschliessen. Aus diesem Grund uebernimmt der Autor keine
; Verantwortung und wird keine daraus folgende oder sonstige Haftung ueber-
; nehmen, die auf irgendeine Art aus der Benutzung dieses Programms oder
; Teilen davon entsteht.
;
; Syntax fuer Metalink ASM51.
;
;==============================================================================
; Konstanten, Variablen, etc.



	$MOD52
	$INCLUDE (intvecs.asm)


; einige Pin-Definitionen

PortRX		bit P3.2
PortSWFreq	bit P3.3
PortLEDError	bit P3.4
PortLEDSync	bit P3.5
PortCharset	bit P3.7
PortSWTime	bit P1.7
PortKeyHHup	bit P1.6
PortKeyHHdn	bit P1.5
PortKeyMMup	bit P1.4
PortKeyMMdn	bit P1.3

; Konstanten

vdly100u	equ 35		; fuer "wait100us" bei fQ = 9.216 MHz
vdly1m		equ 152		; fuer "wait1ms" bei fQ = 9.216 MHz
vTH1		equ 256-5	; T1-Reload fuer 9600 Baud mit SMOD = 1
vTH0		equ 256-240	; T0-Reload fuer 3200 Hz bei fQ = 9.216 MHz

cr		equ 0Dh
lf		equ 0Ah
space		equ 20h
errsymbwin	equ ''		; Fehlersymbol fuer ANSI-Zeichensatz (Windows)
errsymbasc	equ '~'		; Fehlersymbol plain ASCII und DOS
phasig1		equ 70h
phasig2		equ 19h
alfa		equ 70h
ltrs		equ 25h
figs		equ 49h


; Bit-Variablen

	bseg

Shifted:	dbit 1
NewSecond:	dbit 1
RXfull:		dbit 1
TXempty:	dbit 1
Monoflop:	dbit 1
Escape:		dbit 1
ReqShowFreq:	dbit 1
ReqClockSet:	dbit 1
ClockChanged:	dbit 1
KeyhhUp:	dbit 1
KeyhhDown:	dbit 1
KeymmUp:	dbit 1
KeymmDown:	dbit 1
StateSWFreq:	dbit 1
StateSWTime:	dbit 1
BitB:		dbit 1


; Byte-Variablen

	dseg at	24h		; bei 24h ist davor Platz fuer 32 Bit-Variablen

clk32:		ds 1
loopadjust:	ds 1
loopgain:	ds 1
sirawdata:	ds 1
bitsavailable:	ds 1
sibuf:		ds 1
rx:		ds 1
dx:		ds 1
dx1:		ds 1
dx2:		ds 1
rxasc:		ds 1
dx2asc:		ds 1
errsymbol:	ds 1
errors:		ds 1
column:		ds 1
dlycnt:		ds 1
dlycnt2:	ds 1
monotime:	ds 1
cnt3200Hz:	ds 1
cnt100Hz:	ds 1
second:		ds 1
hour:		ds 1
minute:		ds 1
hourset:	ds 1
minuteset:	ds 1
keypattern:	ds 1

stack		equ $



;==============================================================================
; Interrupt-Vektoren

	cseg

	org	VRESET
	ajmp 	start

	org	VEXINT0
	ajmp	exint0

	org	VTIMER0
	ajmp	timer0

	org	VEXINT1
	ajmp	exint1

	org	VSERINT
	ajmp	serint



;==============================================================================
; Hauptprogramm

	org	40h

start:
	mov	SP,#stack
	acall	lamptest
	acall	initit
	setb	EA

	setb	ReqClockSet		; beim Einschalten Uhr stellen
	setb	ReqShowFreq		; und Frequenz anzeigen

mainloop:
	acall	rtcset
	acall	prfreq
	acall	receive
	sjmp	mainloop


txt_hello:	db cr,lf,cr,lf
		db 'DL4BBL''s NAVTEX Decoder -- Version 0.9'
		db cr,lf,cr,lf,0



;==============================================================================
;Interruptroutinen


; exint0: Interruptroutine fuer externen Interrupt 0. Hier findet die
; Synchronisation der Bit-Clock findet hier statt. Das Signal aus dem
; Empfaenger liegt an INT0 an. Im Empfaenger muss dafuer gesorgt sein, dass
; dieser Interrupt "nicht zu oft" auftritt (bei fehlendem Sendersignal etc;
; sollte bei einem entsprechend schmalbandigen Demodulator automatisch der
; Fall sein; im Mustergeraet ist der Abstand immer groesser als 2 ms).


exint0:
	push	ACC
	push	PSW

	mov	A,loopadjust
	add	A,loopgain
	mov	loopadjust,A	; Bitclock nur nachregeln, wenn loopadjust
	jnc	ex0end		;  ueberlaeuft
	
	mov	A,clk32
	mov	C,ACC.4		; Bit 4 von clk32 ist Sitor-Bitclock
	jc	ex0slow
	inc	clk32
	sjmp	ex0end
ex0slow:
	jz	ex0end
	dec	A
	jz	ex0end
	mov	clk32,A
ex0end:
	pop	PSW
	pop	ACC
	reti


;------------------------------------------------------------------------------
; exint1: Interruptroutine fuer externen Interrupt 1. Nicht benutzt.


exint1:
	reti


;------------------------------------------------------------------------------
; timer0: Interruptroutine fuer Timer0. Wird mit einer festen Rate von 3200 Hz
; aufgerufen, also alle 312.5 us.
;
; Hier werden die Daten vom Empfaenger abgeholt: clk32 wird dekrementiert
; und bei clk32 = 0 wird der Zustand des Empfaengerausgangs als neues Bit
; gespeichert.
;
; Bei jeder vollen Sekunde wird die Bit-Variable "NewSecond" gesetzt. Sofern
; eine Echtzeituhr gewuenscht wird, muss deren Verwaltung ausserhalb dieser
; Interruptroutine stattfinden. Das Flag "NewSecond" muss dazu mindestes 1 x
; je Sekunde abgefragt werden.
;
; Schliesslich laeuft ein Software-Timer, der fuer verschiedene
; Zeitmessungen dienen kann und der wie folgt aktiviert wird:
;
;        clr   Monoflop
;        mov   monotime,#gewuenschte_Zeit       ; in 10-ms-Schritten
;        setb  Monoflop
;
; Nach Ablauf der Zeit wird die Bit-Variable "Monoflop" geloescht.


timer0:
	djnz	clk32,ti0RTC

	push	ACC	
	push	PSW
	mov	clk32,#32
	mov	C,PortRX
	mov	A,sirawdata
	rrc	A
	mov	sirawdata,A		; neues Bit von links in sirawdata
	inc	bitsavailable
	pop	PSW
	pop	ACC

ti0RTC:
	djnz	cnt3200Hz,ti0end

	push	ACC
	mov	cnt3200Hz,#32
	djnz	cnt100Hz,ti0nosec
	mov	cnt100Hz,#100
	setb	NewSecond
ti0nosec:
	jnb	Monoflop,ti0exit
	djnz	monotime,ti0exit
	clr	Monoflop
ti0exit:
	pop	ACC
ti0end:
	reti


;------------------------------------------------------------------------------
; serint: Interruptroutine fuer die serielle Schnittstelle.


serint:
	jnb	RI,sertx
	clr	RI
	setb	RXfull
sertx:
	jnb	TI,serexit
	clr	TI
	setb	TXempty
serexit:
	reti



;==============================================================================
; Routinen fuer SITOR-Empfang



; receive: Gesamten SITOR-B-Empfang abwickeln einschliesslich Einphasen.


receive:
	setb	PortLEDSync
	setb	PortLEDError
	mov	loopgain,#50
	mov	loopadjust,#255
	clr	Shifted
	clr	Escape
	
	clr	EA
	mov	sirawdata,#0
	mov	bitsavailable,#0
	setb	EA

; Synchronisation: Auf Phasing-Sequenz rx/dx/rx = 1-2-1 warten

syncloop:
	acall	getonebit
	jb	Escape,recend
	cjne	A,#phasig1,syncloop
	acall	get7bits
	jb	Escape,recend
	cjne	A,#phasig2,syncloop
	mov	dx,A
	acall	get7bits
	jb	Escape,recend
	cjne	A,#phasig1,syncloop

	mov	loopgain,#10
	mov	errors,#0
	mov	dx1,#0
	clr	PortLEDSync
	mov	A,#lf
	acall	prtty

; Synchronisiert. Ab hier jeweils 2 Zeichen empfangen, dx und rx

recloop:
	mov	dx2,dx1		; dx2/dx1/dx: "FIFO" fuer 3 Zeichen
	mov	dx1,dx
	acall	get7bits
	mov	dx,A
	acall	get7bits
	jb	Escape,recend
	setb	PortLEDError
	mov	rx,A
	xrl	A,#phasig1
	jz	recloop			; Sprung, wenn Idling

; empfangenes Zeichen wandeln und ausgeben, falls darstellbar

	acall	convertrx
	acall	prtty

; Bei "end-of-emission signal" (alfa in Position dx) abbrechen und auf neue
; Einphas-Sequenz warten. Fehlerzaehler pruefen. Bei zu vielen gestoerten
; Zeichen hintereinander ebenfalls abbrechen und neu einphasen

	mov	A,dx2
	cjne	A,#alfa,checkerrors
	ajmp	receive
checkerrors:
	mov	A,errors
	clr	C
	subb	A,#12
	jc	recloop
	ajmp	receive
recend:
	setb	PortLEDSync
	setb	PortLEDError
	ret


;------------------------------------------------------------------------------
; convertrx: rx in ASCII-Zeichen wandeln, Ergebnis in A. LTRS/FIGS-Umschaltung
; auswerten. Puefen, ob in rx gueltiger Code empfangen wurde. Falls rx
; gueltig: wenn dx2 auch gueltig, muss dx2 = rx sein, sonst Fehler. Falls rx
; ungueltig: durch dx2 ersetzen. Fehler, wenn dx2 auch ungueltig. In Routine
; "receive" benutzt.


convertrx:
	mov	A,dx2
	acall	sitoasc
	mov	dx2asc,A
	mov	A,rx
	acall	sitoasc
	mov	rxasc,A
	jz	rxmutilated

	mov	A,dx2asc
	jz	testltrs	; Sprung, wenn rx ok und dx2 fehlerhaft
	mov	A,rx
	xrl	A,dx2
	jz	testltrs	; Sprung, wenn rx ok und gleich dx2
	sjmp	bothdifferent	; Sprung, wenn beide ok, aber verschieden
rxmutilated:
	clr	PortLEDError
	mov	rx,dx2
	mov	A,dx2asc
	mov	rxasc,A
	jz	bothmutilated	; Sprung wenn rx fehlerhaft und dx2 auch
testltrs:
	mov	A,rx
	cjne	A,#ltrs,testfigs
	clr	Shifted
	sjmp	convrxend
testfigs:
	cjne	A,#figs,convrxend
	setb	Shifted
	sjmp	convrxend
bothmutilated:
bothdifferent:
	clr	PortLEDError
	mov	rxasc,#0
convrxend:
	mov	A,rxasc
	ret


;------------------------------------------------------------------------------
; prtty: ASCII-Zeichen in A ausgeben, in dieser Version via RS-232. LF als
; CR LF ausgeben, CR ignorieren. Fehlerzaehler "errors" erhoehen, wenn A = 0;
; dann Fehlersymbol ausgeben. Am Zeilenanfang Uhrzeit ausgeben, wenn Schalter
; dafuer geschlossen.


prtty:
	jnz	prttynoerr
	inc	errors
	mov	R3,errsymbol
	sjmp	prttynormal
prttynoerr:
	mov	R3,A
	dec	errors
	mov	A,errors
	jnb	ACC.7,prttychk
	mov	errors,#0
prttychk:
	mov	A,R3
	xrl	A,#lf
	jz	prttynewline
	mov	A,R3
	clr	C
	subb	A,#20h
	jc	prttyend
prttynormal:
	mov	A,R3
	acall	sendchar
	mov	A,column
	subb	A,#79
	jnc	prttynewline		; harter Umbruch am Zeilenende
	mov	A,column
	clr	C
	subb	A,#72
	jc	prttyend
	mov	A,R3
	xrl	A,errsymbol
	jz	prttynewline		; Umbruch, falls Space oder
	mov	A,R3			;   Fehlersymbol fast am Zeilenende
	xrl	A,#space
	jnz	prttyend
prttynewline:
	acall	prlinefeed
	jb	PortSWTime,prttyend
	acall	prtime
	mov	A,#space
	acall	sendchar	
prttyend:
	ret


;------------------------------------------------------------------------------
; getonebit: Ein Bit empfangen
; get7bits:  7 Bits empfangen
;
; Die Interruptroutine fuer Timer 0 schiebt die empfangenen Bits von links
; in sirawdata und erhoeht bei jedem Bit den Zaehler bitsavailable um 1.
; getonebit wartet auf bitsavailable<>0, holt ein Bit ab und dekrementiert
; bitsavailable. Meistens wird bitsavailable = 1 sein, dann wird Bit 7 von
; rawdata abgeholt. Falls sich das Abholen verzoegert und inzwischen weitere
; neue Daten aufgelaufen sind, holt getonebit das naechste Bit entsprechend
; aus Bit 6, 5, usw. von sirawdata. Insgesamt koennen 8 Bits gepuffert
; werden (= 80 ms).
;
; Wenn an der Stelle getone1 das Idle-Bit in PCON gesetzt wird, sinkt der
; Stromverbrauch des Empfaengers von etwa 30 auf 22 mA. Allerdings sind
; dann im Kopfhoerer helle Geraeusche hoerbar, wie auch bei der RS-232-
; Ausgabe. Dies sind keine Einstreuungen in den Empfaengereingang, sondern
; der NF-Verstaerker TDA7052 faengt sich transiente Signale vom uC ein.
;
; Veraenderte Register: A, PSW (C), R3, R4


getonebit:
	acall	rtcupdate
	acall	checkesc
	jb	Escape,getonex	
getone1:
	mov	A,bitsavailable
	jz	getone1
	clr	EA
	mov	A,sirawdata
	mov	R3,bitsavailable
	dec	bitsavailable
	setb	EA
getone2:
	rlc	A			; bitsavailable bestimmt, welches Bit
	djnz	R3,getone2		;  aus sirawdata zu holen ist. 
	jb	PortSWFreq,getone3	;  1 -> Bit 7, 2 -> Bit 6, ...
	cpl	C
getone3:
	mov	A,sibuf
	rrc	A
	mov	sibuf,A
	anl	A,#7Fh
getonex:
	ret


get7bits:
	mov	R4,#7
get7loop:
	acall	getonebit
	jb	Escape,get7end
	djnz	R4,get7loop
get7end:
	ret


;------------------------------------------------------------------------------
; sitoasc: Sitor- in ASCII-Zeichen wandeln


sitoasc:
	mov	C,Shifted
	mov	ACC.7,C
	mov	DPTR,#sitable
	movc	A,@A+DPTR
	ret

; In der folgenden Tabelle bedeutet 0 einen nicht benutzten Code. Wenn solch
; ein Code empfangen wurde, war der Empfang gestoert. 1 bedeutet ein nicht
; darstellbares Zeichen, z.B. LTRS.

sitable:
	db  0,  0,  0,  0,  0,  0,  0, cr,  0,  0,  0,'t',  0,'b','o',  0
	db  0,  0,  0, lf,  0,  1,'h',  0,  0,  0,'l',  0,'z',  0,  0,  0
	db  0,  0,  0,' ',  0,  1,'n',  0,  0,'e','r',  0,'d',  0,  0,  0
	db  0,'u','i',  0,'s',  0,  0,  0,'a',  0,  0,  0,  0,  0,  0,  0
	db  0,  0,  0,'v',  0,'x','m',  0,  0,  1,'g',  0,  1,  0,  0,  0
	db  0,'q','p',  0,'y',  0,  0,  0,'w',  0,  0,  0,  0,  0,  0,  0
	db  0,'k','c',  0,'f',  0,  0,  0,'j',  0,  0,  0,  0,  0,  0,  0
	db  1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
	db  0,  0,  0,  0,  0,  0,  0, cr,  0,  0,  0,'5',  0,'?','9',  0
	db  0,  0,  0, lf,  0,  1,'#',  0,  0,  0,')',  0,'+',  0,  0,  0
	db  0,  0,  0,' ',  0,  1,2Ch,  0,  0,'3','4',  0,'$',  0,  0,  0
	db  0,'7','8',  0,27h,  0,  0,  0,'-',  0,  0,  0,  0,  0,  0,  0
	db  0,  0,  0,'=',  0,'/','.',  0,  0,  1,'&',  0,  1,  0,  0,  0
	db  0,'1','0',  0,'6',  0,  0,  0,'2',  0,  0,  0,  0,  0,  0,  0
	db  1,'(',':',  0,'!',  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0
	db  1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0


;------------------------------------------------------------------------------
; checkesc: An Schaltern und Tasten pruefen, ob der Empfang beendet werden
; soll. Das kann die Betaetigung des Frequenzschalters sein (= Frequenz
; anzeigen und neu synchronisieren), oder ein Druck auf alle Uhrenstelltasten
; gleichzeitig (geht mit dem Daumen, = Uhr soll gestellt werden).
;
; Bei Abbruch wird die Bit-Variable Escape = 1 gesetzt sowie ReqClockSet = 1,
; wenn "Uhr stellen" angefordert wurde und ReqShowFreq = 1 bei Frequenzwechsel.
;
; Das Flag Escape muss nach jedem Aufruf von getonebit oder get7bits abgefragt
; werden. get7bits liefert, wenn Escape = 1 ist, keine Daten, sondern bricht
; ebenfalls vorzeitig ab.


checkesc:
	mov	A,P1
	anl	A,#01111000b
	jnz	checkesc1
	setb	Escape
	setb	ReqClockSet
checkesc1:
	mov	C,PortSWFreq
	jnb	StateSWFreq,checkesc2
	cpl	C
checkesc2:
	mov	ReqShowFreq,C
	orl	C,Escape
	mov	Escape,C
	ret



;==============================================================================
; Verschiedene Routinen



; prfreq: Trennline mit eingestellter Frequenz ausgeben (490 oder 518 kHz),
; wenn ReqShowFreq = 1 oder wenn sich die Schalterstellung (in StateSWFreq
; aufbewahrt) geaendert hat.


prfreq:
	jbc	ReqShowFreq,prfreq1
	mov	C,PortSWFreq
	mov	BitB,C
	mov	C,StateSWFreq
	acall	xorbits
	jnc	prfreqend
prfreq1:
	acall	prnewline
	acall	prlinefeed
	mov	C,PortSWFreq
	mov	StateSWFreq,C
	mov	DPTR,#txt_490
	jnc	prfreq2
	mov	DPTR,#txt_518
prfreq2:
	mov	R2,DPL
	mov	R3,DPH
	mov	R4,#3
prfreq4:
	mov	DPL,R2
	mov	DPH,R3
	acall	prctext
	djnz	R4,prfreq4
	
	mov	DPTR,#txt_asx
	acall	prctext
prfreqend:
	ret

txt_490:	db '********** 490 kHz '
		db 0
txt_518:	db '********** 518 kHz '
		db 0
txt_asx:	db '**********'
		db cr,lf,0



;------------------------------------------------------------------------------
; rtcupdate: Uhrzeit um eine Sekunde erhoehen. Muss mindestens 1 x je Sekunde
; aufgerufen werden (die Uhrzeit wird nicht vom Timer-Interrupt verwaltet,
; sondern nur das Flag "NewSecond").


rtcupdate:
	jnb	NewSecond,rtcupex
	clr	NewSecond
	mov	A,second
	add	A,#1
	da	A
	mov	second,A
	cjne	A,#60h,rtcupex
	mov	second,#0
	mov	A,minute
	add	A,#1
	da	A
	mov	minute,A
	cjne	A,#60h,rtcupex
	mov	minute,#0
	mov	A,hour
	add	A,#1
	da	A
	mov	hour,A
	cjne	A,#24h,rtcupex
	mov	hour,#0
rtcupex:
	ret


;------------------------------------------------------------------------------
; rtcset: Uhrzeit stellen. 


rtcset:
	jbc	ReqClockSet,rtcset1
	ajmp	rtcsetexit
rtcset1:
	mov	C,PortSWTime
	mov	StateSWTime,C
	clr	ClockChanged
	mov	dlycnt,#200
	mov	DPTR,#txt_clkset
	acall	prctext
	acall	prtime
	mov	hourset,hour
	mov	minuteset,minute
rtcsetloop1:
	acall	prlinefeed
	mov	R2,#12
rtcsetloop2:					; auf Loslassen warten
	acall	rtcupdate
	acall	keyboard
	jnz	rtcsetloop2
rtckeywait:					; auf Tastendruck warten
	mov	C,StateSWTime
	mov	BitB,C
	mov	C,PortSWTime			; Uhr stellen beenden durch
	acall	xorbits				;   Umlegen des TIME STAMP-
	jc	rtcfinished			;   Schalters
	acall	rtcupdate
	acall	keyboard
	jnz	rtckeyeval
	jb	ClockChanged,rtckeywait		; erster Tastendruck muss
	djnz	dlycnt,rtckeywait		;   innerhalb von etwa 10 s
	sjmp	rtcfinished			;   erfolgen, sonst Abbruch
rtckeyeval:
	setb	ClockChanged
	jb	KeyhhUp,rtc_hhup
	jb	KeyhhDown,rtc_hhdn
	jb	KeymmUp,rtc_mmup
	jb	KeymmDown,rtc_mmdn
rtc_hhup:
	mov	A,hourset
	add	A,#1
	da	A
	mov	hourset,A
	cjne	A,#24h,rtcset2
	mov	hourset,#00h
	sjmp	rtcset2
rtc_hhdn:
	mov	A,hourset
	add	A,#99h
	da	A
	mov	hourset,A
	cjne	A,#99h,rtcset2
	mov	hourset,#23h
	sjmp	rtcset2
rtc_mmup:
	mov	A,minuteset
	add	A,#1
	da	A
	mov	minuteset,A
	cjne	A,#60h,rtcset2
	mov	minuteset,#00h
	sjmp	rtcset2
rtc_mmdn:
	mov	A,minuteset
	add	A,#99h
	da	A
	mov	minuteset,A
	cjne	A,#99h,rtcset2
	mov	minuteset,#59h
rtcset2:
	mov	R0,#hourset
	acall	prtimevec
	djnz	R2,rtcsetloop2
	sjmp	rtcsetloop1

rtcfinished:
	jb	ClockChanged,rtcchanged
	acall	prnewline
	mov	DPTR,#txt_clkunc
	acall	prctext
	sjmp	rtcsetend
rtcchanged:
	mov	hour,hourset
	mov	minute,minuteset
	mov	second,#10
	mov	A,#55
	acall	prnlif
	mov	DPTR,#txt_clkfin
	acall	prctext
rtcsetend:
	acall	prtime
	acall	prlinefeed
rtcsetexit:
	ret


txt_clkset:	db cr,lf,cr,lf
		db 'Use receiver keys to set clock,'
		db ' Time Stamp switch to exit. Now '
		db 0

txt_clkfin:	db 'Clock set to '
		db 0

txt_clkunc:	db 'Setting not changed. Now '
		db 0


;------------------------------------------------------------------------------
; keyboard: Die 4 Tasten fuer das Stellen der Uhr entprellt abfragen, aber
; nicht auf Tastendruck warten. Nach dem Aufruf koennen die gedrueckten
; Tasten an ACC.6 ... ACC.3 abgelesen werden sowie in den Bit-Variablen
; KeyhhUp, KeyhhDown, KeymmUp und KeymmDown. Taste gedrueckt = 1.


keyboard:
	mov	A,P1
	cpl	A
	anl	A,#01111000b
keyb1:
	mov	keypattern,A
	clr	Monoflop
	mov	monotime,#5
	setb	Monoflop
keyb2:
	mov	A,P1
	cpl	A
	anl	A,#01111000b
	cjne	A,keypattern,keyb1
	jb	Monoflop,keyb2
	mov	C,ACC.6
	mov	KeyhhUp,C
	mov	C,ACC.5
	mov	KeyhhDown,C
	mov	C,ACC.4
	mov	KeymmUp,C
	mov	C,ACC.3
	mov	KeymmDown,C
	ret


;------------------------------------------------------------------------------
; prtime: Zeit als hh:mm ausgeben, dahinter 1 x Leerzeichen.


prtime:
	mov	R0,#hour
prtimevec:
	mov	A,@R0
	inc	R0
	acall	prhexbyte
	mov	A,#':'
	acall	sendchar
	mov	A,@R0
	inc	R0
	acall	prhexbyte
	mov	A,#space
	acall	sendchar
	ret


;------------------------------------------------------------------------------
; prlinefeed:  CR LF an RS-232 ausgeben.
;
; prnewline:   CR LF an RS-232 ausgeben, wenn Cursor nicht in Spalte 0.
;
; prnlif:      CR LF nur, wenn Spalte > Wert in A


prnlif:
	clr	C
	subb	A,column
	jc	prlinefeed
	sjmp	prlfend
	
prnewline:
	mov	A,column
	jnz	prlinefeed
	sjmp	prlfend
	
prlinefeed:
	mov	A,#cr
	acall	sendchar
	mov	A,#lf
	acall	sendchar
prlfend:
	ret


;------------------------------------------------------------------------------
; prctext: ASCII-Zeichenkette aus Programmspeicher an RS-232 ausgeben. Vor
; dem Aufruf muss DPTR auf das erste Zeichen zeigen. Die Zeichenkette muss
; mit 0 abgeschlossen sein.
;
; Veraenderte Register: PSW, A, DPTR


prctext:
	clr	A
	movc	A,@A+DPTR
	jz	prcexit
	jnb	TXempty,$
	clr	TXempty
	mov	SBUF,A
	inc	DPTR
;;;	acall	idle
	sjmp	prctext
prcexit:
	ret


;------------------------------------------------------------------------------
; prstring: ASCII-Zeichenkette aus Datenspeicher an RS-232 ausgeben. Vor dem
; Aufruf muss R1 auf das erste Zeichen zeigen. Die Zeichenkette muss mit 0
; abgeschlossen sein.
;
; Veraenderte Register: PSW, A, R1


prstring:
	mov	A,@R1
	jz	prstexit
	jnb	TXempty,$
	clr	TXempty
	mov	SBUF,A
	inc	R1
;;;	acall	idle
	sjmp	prstring
prstexit:
	ret


;------------------------------------------------------------------------------
; prhexbyte: Byte als zweistellige Hexzahl an RS-232 ausgeben. Wird z.B.
; fuer die Ausgabe der Uhrzeit benoetigt.


prhexbyte:
	push	ACC
	swap	A
	acall	digasc
	acall	sendchar
	pop	ACC
	acall	digasc
	acall	sendchar
	ret

digasc:
	anl	A,#0Fh
	clr	C
	subb	A,#0Ah
	jc	digasc2
	add	a,#7
digasc2:
	add	A,#3Ah
	ret


;------------------------------------------------------------------------------
; sendchar: Ein Byte an RS-232 ausgeben. column um 1 erhoehen; bei CR
; auf 0 setzen.


sendchar:
	jnb	TXempty,$
	clr	TXempty
	mov	SBUF,A
	cjne	A,#lf,sendch1
	sjmp	sendchx
sendch1:
	inc	column
	cjne	A,#cr,sendchx
	mov	column,#0
sendchx:
	ret


;------------------------------------------------------------------------------
; sendbyte: Ein Byte an RS-232 ausgeben.


sendbyte:
	jnb	TXempty,$
	clr	TXempty
	mov	SBUF,A
	ret


;------------------------------------------------------------------------------
; xorbits: Bit-Exklusiv-Oder. Bit 1 in C, Bit 2 in BitB, Ergebnis in C.


xorbits:
	jnb	BitB,xorbitsx
	cpl	C
xorbitsx:
	ret


;------------------------------------------------------------------------------
; idle: Prozessor in idle mode schalten. Senkt Stromverbrauch etwa auf 1/4.


idle:
	mov	A,PCON
	setb	ACC.0
	mov	PCON,A
	ret


;------------------------------------------------------------------------------
; initit: Interrupts, serielle Schnittstelle, Timer und Variablen
; initialisieren. Timer 1 erzeugt die Baudrate fuer die serielle Schnittstelle.


initit:
	mov	A,#errsymbwin
	jb	PortCharset,initit2
	mov	A,#errsymbasc
initit2:
	mov	errsymbol,A
	mov	cnt3200Hz,#32
	mov	cnt100Hz,#1
	clr	A
	mov	minute,A
	mov	hour,A
	mov	loopgain,A
	mov	column,A
	mov	second,#15

	clr	RXfull
	setb	TXempty
	
	setb	IT0		; Triggermodus EXINT0: fallende Flanke
	clr	PX0		; Interrupt-Prioritaet EXINT0: niedrig

	clr	PS			; Prioritaet ser. Port: niedrig
	mov	SCON,#01000000b		; Serial Mode = 1, reception disabled
	mov	PCON,#10000000b		; SMOD = 1

	mov     TMOD,#00100010b		; Timer 0 & 1: 8 Bit Auto Reload
	mov	TH1,#vTH1
	setb	TR1
	
	setb	PT0		; Interrupt-Prioritaet Timer 0: hoch
	mov	TH0,#vTH0
	setb	TR0
	
	setb	EX0		; externen Interrupt 0 freigeben
	setb	ET0		; Interrupt Timer 0 freigeben
	setb	ES		; Interrupt serieller Port freigeben
	ret


;------------------------------------------------------------------------------
; lamptest: Alle LEDs ein- und ausschalten.


lamptest:
	clr	PortLEDSync
	clr	PortLEDError
	acall	wait100ms
	setb	PortLEDSync
	setb	PortLEDError
	acall	wait100ms
	clr	PortLEDSync
	clr	PortLEDError
	acall	wait100ms
	setb	PortLEDSync
	setb	PortLEDError
	ret


;------------------------------------------------------------------------------
; wait100us: Etwa 100 Mikrosekunden vertroedeln.
; wait1ms:   Etwa 1 Millisekunde vertroedeln.
; wait10ms:  Etwa 10 Millisekunden vertroedeln.
; wait100ms: Etwa 100 Millisekunden vertroedeln.
;
; Alle Zeiten nur sehr grob. Funktioniert natuerlich nicht im idle mode.


wait100us:
	mov	dlycnt,#vdly100u	; T = 24 * vdly100u + 72
	djnz	dlycnt,$
	ret

wait10ms:
	mov	dlycnt2,#10
	sjmp	wait1ma

wait100ms:
	mov	dlycnt2,#100
	sjmp	wait1ma

wait1ms:
	mov	dlycnt2,#1
wait1ma:
	mov	dlycnt,#vdly1m		; T = dlycnt2 * (60 * vdly1m + 120)
wait1mb:
	nop
	nop
	nop
	djnz	dlycnt,wait1mb
	djnz	dlycnt2,wait1ma
	ret



;==============================================================================

	END
